Package: dendroTools
Type: Package
Title: Linear and Nonlinear Methods for Analyzing Daily and Monthly
        Dendroclimatological Data
Version: 1.2.13
Author: Jernej Jevsenak [aut, cre]
Maintainer: Jernej Jevsenak <jernej.jevsenak@gmail.com>
Description: Provides novel dendroclimatological methods, primarily used by the
    Tree-ring research community. There are four core functions. The first one is 
    daily_response(), which finds the optimal sequence of days that are related 
    to one or more tree-ring proxy records. Similar function is daily_response_seascorr(), 
    which implements partial correlations in the analysis of daily response functions.
    For the enthusiast of monthly data, there is monthly_response() function.
    The last core function is compare_methods(), which effectively compares several 
    linear and nonlinear regression algorithms on the task of climate reconstruction.   
License: GPL-3
URL: https://github.com/jernejjevsenak/dendroTools
BugReports: https://github.com/jernejjevsenak/dendroTools/issues
Encoding: UTF-8
LazyData: true
Suggests: testthat, rmarkdown
RoxygenNote: 7.1.2
Imports: ggplot2(>= 2.2.0), brnn(>= 0.6), reshape2(>= 1.4.2), scales(>=
        0.4.1), stats, oce(>= 1.2-0), MLmetrics(>= 1.1.1), dplyr(>=
        0.7.0), knitr(>= 1.19), magrittr(>= 1.5), plotly(>= 4.7.1),
        randomForest(>= 4.6-14), Cubist(>= 0.2.2), lubridate (>=
        1.7.4), psych (>= 1.8.3.3), boot(>= 1.3-22), viridis (>=
        0.5.1), dplR (>= 1.7.2)
Depends: R(>= 3.4)
NeedsCompilation: no
Repository: CRAN
VignetteBuilder: knitr
Packaged: 2024-04-26 05:46:50 UTC; JernejJ
Date/Publication: 2024-04-26 10:10:04 UTC
Built: R 4.2.3; ; 2024-04-26 11:26:02 UTC; unix
