export(daily_response)
export(plot_extreme)
export(plot_heatmap)
export(critical_r)
export(count_ones)
export(years_to_rownames)
export(calculate_metrics)
export(compare_methods)
export(smooth_matrix)
export(round_df)
export(boot_f_cor)
export(boot_f_pcor)
export(boot_f_lm)
export(boot_f_brnn)
export(round2)
export(glimpse_daily_data)
export(data_transform)
export(monthly_response)
export(daily_response_seascorr)
export(monthly_response_seascorr)

S3method(summary, "dmrs")
S3method(plot, "dmrs")

import(ggplot2)
import(viridis)
importFrom("brnn", "brnn", "predict.brnn")
importFrom("reshape2", "melt", "dcast")
importFrom("scales", "pretty_breaks")
importFrom("stats", "cor", "lm", "qt", "quantile", "predict", "sd", "as.formula", "princomp", "model.matrix", "median", "step", "cor.test", "loess.smooth", "residuals")
importFrom("utils", "capture.output", "setTxtProgressBar", "txtProgressBar")
importFrom("graphics", "plot")
importFrom("MLmetrics", "RRSE", "RMSE")
importFrom("Cubist", "cubist", "cubistControl")
importFrom("dplyr", "select", "desc", "summarise", "filter", "group_by")
importFrom("oce", "rescale")
importFrom("knitr", "kable")
importFrom("magrittr", "%>%")
importFrom("plotly", "plot_ly", "add_surface", "layout", "subplot", "plotly_build", "colorbar", "add_trace")
importFrom("randomForest", "randomForest")
importFrom("lubridate", "yday", "year", "ymd", "month", "ydm", "mdy", "myd", "dmy", "dym")
importFrom("psych", "partial.r", "corr.p")
importFrom("boot", "boot", "boot.ci")
importFrom("dplR", "detrend")
