#' Tree-ring width (TRW) data from 2012 - 1961
#'
#' A dataset of tree-ring widths (TRW) from a site in Krakovo forest (Slovenia).
#' The first row represents a value of a year in 1961.
#'
#' @format A data frame with 52 rows and 1 variable:
#' \describe{
#'   \item{TRW}{Standardized tree-ring width indices from 2012 - 1961}
#' }
#' @source Tom Levanič, Slovenian Forestry Institute, Večna pot 2,
#' Ljubljana, Slovenia
#' @export
"data_TRW_1"
