#' Monthly sums of precipitation for Ljubljana from 2018 - 1900. Tidy format.
#'
#' A dataset of monthly sums of precipitations in Ljubljana (Slovenia). The first row
#' represents precipitation sum for January 1900.
#'
#' @format A data frame with 1417 rows and 3 variables:
#' \describe{
#'   \item{Year}{year}
#'   \item{Month}{Month}
#'   \item{Precipitation}{Sum of precipitation}
#' }
#' @source \url{http://climexp.knmi.nl/start.cgi}
#' @export
"LJ_monthly_precipitation"
