#' Example of dataset as required for compare_methods()
#'
#' A dataset of Mean Vessel Area (MVA) tree-ring parameter from a lowland
#' forest in Slovenia. The first row represents a value of a year in 2012.
#'
#' @format A data frame with 58 rows and 3 columns :
#' \describe{
#'   \item{MVA}{Mean Vessel Area measurements from 2012 - 1955}
#'   \item{T_APR}{Mean April temperatures from 2012 - 1955}
#'   \item{T_aug_sep}{Mean August-September temperatures from preceding
#'   growing season from 2012 - 1955}
#' }
#' @source Jernej Jevšenak, Slovenian Forestry Institute, Večna pot 2,
#' Ljubljana, Slovenia
#' @export
"example_dataset_1"
