#' Daily precipitation for Ljubljana from 2017 - 1900
#'
#' A dataset of daily sum of precipitation [mm] in Ljubljana (Slovenia).
#' The first row represents precipitation in 1900 on DOY 1.
#'
#' @format A data frame with 43067 rows and 3 variables:
#' \describe{
#'   \item{Year}{year}
#'   \item{DOY}{day of year}
#'   \item{Precipitation}{Sum of precipitation in mm}
#' }
#' @source \url{http://climexp.knmi.nl/start.cgi}
#' @export
"LJ_daily_precipitation"
