#' Standardised tree-ring width chronology swit272, Larix decidua Mill.
#'
#' A TRW chronology swit272
#' Investigators: Bigler, C.; Claluna, A.
#' Site_Name:  Sils-Maria GR Blais dal Fo
#' Location: Switzerland
#' Northernmost_Latitude: 46.4333
#' Southernmost_Latitude: 46.4333
#' Easternmost_Longitude: 9.7833
#' Westernmost_Longitude: 9.7833
#' Elevation: 2100
#'
#' @format A data frame with 273 rows and 1 variable:
#' \describe{
#'   \item{TRWi}{Standardised tree-ring width chronology}
#' }
#' @source \url{https://www.ncei.noaa.gov/access/paleo-search/study/14108}
#' @export
"swit272"
