% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_heatmap.R
\name{plot_heatmap}
\alias{plot_heatmap}
\title{plot_heatmap}
\usage{
plot_heatmap(
  result_daily_response,
  reference_window = "start",
  type = "daily",
  show_title = FALSE,
  show_year_separators = TRUE
)
}
\arguments{
\item{result_daily_response}{a list with objects as produced by
\code{\link{daily_response}}, \code{\link{monthly_response}},
\code{\link{daily_response_seascorr}} or
\code{\link{monthly_response_seascorr}}.}

\item{reference_window}{character string, the reference_window argument describes
how each calculation is referred. Options are 'start', 'end' and 'middle'.
If available, the value stored in result_daily_response$reference_window is used.}

\item{type}{the character string describing type of analysis: daily or monthly.}

\item{show_title}{logical. If TRUE, the plot title with analysed period, method
and reference-window information is shown. The default is FALSE.}

\item{show_year_separators}{logical. If TRUE, dashed vertical lines are
added between relative-year blocks in plots with previous-year data.
The default is TRUE.}
}
\value{
A ggplot2 object containing the heatmap display
}
\description{
Graphs a heatmap of values stored in a matrix, such as produced
by \code{\link{daily_response}} or \code{\link{monthly_response}} functions.
}
\keyword{internal}
