% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_TRW.R
\docType{data}
\name{data_TRW}
\alias{data_TRW}
\title{Tree-ring width (TRW) example proxy from 1981 - 1757}
\format{
A data frame with 225 rows and 1 variable:
\describe{
  \item{TRW}{residual TRW indices from 1981 - 1757}
}
}
\source{
\itemize{
   \item Schweingruber, F.H., 1981. Vrsic Krajnska Gora - PCAB - ITRDB YUGO001.
   \item https://www.ncei.noaa.gov/access/paleo-search/study/4728}
}
\usage{
data_TRW
}
\description{
A dataset with TRW proxy records from a site in Slovenian Alps - Vrsic.
The first row represents a TRW value in a year 1757. Row names represent
years.
}
\keyword{datasets}
